//There are a number of things that "ordered set" can mean; this implements the one people usually want, which is basically an array with element uniquing and O(1) contains() and indexOf()
public struct MyOrderedSet<Element : Hashable> : ArrayLiteralConvertible, MutableCollectionType, RangeReplaceableCollectionType {
    //Array and Dictionary take care of all the bounds checking and copy-on-write stuff for us. Nice.
    private var _indices:[Element : Int] = [:] //map values to the indices that they're stored at in the _values array
    private var _values:[Element] = [] //the actual values
    
    public var count : Int {
        return _values.count
    }
    
    public var startIndex : Int {
        return _values.startIndex //always going to be 0, but for symmetry's sake, may as well say it this way
    }
    
    public var endIndex : Int {
        return _values.endIndex
    }
    
    public init() {}
    
    public init<Sequence:SequenceType where Sequence.Generator.Element == Element>(elements: Sequence) {
        replaceRange(self.startIndex..<self.endIndex, with: elements)
    }
    
    public init(arrayLiteral elements: Element...) {
        self.init(elements:elements)
    }
    
    public func indexOf(value:Element) -> Int? { //override the protocol extension version for performance, since we can do better
        return _indices[value]
    }
    
    public func contains(element: Element) -> Bool { //override the protocol extension version for performance, since we can do better
        return _indices[element] != nil
    }
    
    public mutating func remove(element:Element) -> Element? {
        if let index = indexOf(element) {
            replaceRange(index...index, with: EmptyCollection())
            return element
        }
        return nil
    }
    
    private mutating func removeElementsInRange(subRange: Range<Int>) {
        for value in _values[subRange] {
            _indices[value] = nil
        }
        _values.replaceRange(subRange, with: EmptyCollection())
    }
    
    private mutating func addElements<Sequence:SequenceType where Sequence.Generator.Element == Element>(newElements: Sequence, at index:Int) -> Int {
        var addedElements = 0
        
        for newValue in newElements {
            if indexOf(newValue) == nil { //I've decided to say that the first occurrence of a value that's added "wins". Could do the reverse by removing the element if already present, but that would mean recalculating the current index, and I'm lazy
                _indices[newValue] = index + addedElements
                _values.insert(newValue, atIndex: index + addedElements)
                addedElements++
            }
        }
        
        return addedElements
    }
    
    //When we add or remove elements, we have to adjust the element->index mapping for the new indices of all the elements that moved
    private mutating func shiftIndicesStartingAt(index:Int, by distance:Int) {
        for value in _values.suffixFrom(index) {
            _indices[value]! += distance
        }
    }
    
    public mutating func replaceRange<Sequence:SequenceType where Sequence.Generator.Element == Element>(subRange: Range<Int>, with newElements: Sequence) {
        //Step one: remove everything in the range being replaced
        removeElementsInRange(subRange)
        
        //Step two: add the new elements at the start of the range
        let addedElementsCount = addElements(newElements, at: subRange.startIndex)
       
        //Step three: adjust our dictionary of elements to indices to reflect the new positions of things
        shiftIndicesStartingAt(subRange.startIndex + addedElementsCount, by: addedElementsCount - subRange.count)
    }
    
    public subscript(index: Int) -> Element {
        get {
            return _values[index]
        }
        set {
            replaceRange(index...index, with: CollectionOfOne(newValue))
        }
    }
}

var testSet:MyOrderedSet<String> = ["weasel", "frog", "magnet", "ectoplasm", "frog", "weasel"]
print(testSet)
print(Array(testSet))
print(Array(testSet[1...2]))
testSet.append("glow worm")
var uppercaseSet = MyOrderedSet(elements: testSet.map { $0.uppercaseString })
print(Array(uppercaseSet))
uppercaseSet.remove("WEASEL")
uppercaseSet.remove("FROG")
print(Array(uppercaseSet))
uppercaseSet[1] = "WOMBAT"
print(Array(uppercaseSet))
print(uppercaseSet.sort())

